@@ -190,7 +190,7 @@ couchTests.changes = function(debug) {
     _id : "_design/changes_filter",
     "filters" : {
       "bop" : "function(doc, req) { return (doc.bop);}",
-      "dynamic" : stringFun(function(doc, req) { 
+      "dynamic" : stringFun(function(doc, req) {
         var field = req.query.field;
         return doc[field];
       }),
@@ -205,6 +205,13 @@ couchTests.changes = function(debug) {
     views : {
       local_seq : {
         map : "function(doc) {emit(doc._local_seq, null)}"
+      },
+      blah: {
+        map : 'function(doc) {' +
+              '  if (doc._id == "blah") {' +
+              '    emit(doc._id, null);' +
+              '  }' +
+              '}'
       }
     }
   };
@@ -213,7 +220,7 @@ couchTests.changes = function(debug) {
 
   var req = CouchDB.request("GET", "/test_suite_db/_changes?filter=changes_filter/bop");
   var resp = JSON.parse(req.responseText);
-  T(resp.results.length == 0); 
+  T(resp.results.length == 0);
 
   db.save({"bop" : "foom"});
   db.save({"bop" : false});
@@ -319,7 +326,16 @@ couchTests.changes = function(debug) {
   var resp = JSON.parse(req.responseText);
   var expect = (!is_safari && xhr) ? 3: 1;
   TEquals(expect, resp.results.length, "should return matching rows");
-  
+ 
+  // test filter on view function (map)
+  //
+  T(db.save({"_id":"blah", "bop" : "plankton"}).ok);
+  var req = CouchDB.request("GET", "/test_suite_db/_changes?filter=_view&view=changes_filter/blah");
+  var resp = JSON.parse(req.responseText);
+  T(resp.results.length === 1);
+  T(resp.results[0].id === "blah");
+
+
   // test for userCtx
   run_on_modified_server(
     [{section: "httpd",
@@ -374,8 +390,11 @@ couchTests.changes = function(debug) {
       filters: {
         foo:
           'fun({Doc}, Req) -> ' +
-          '  Value = couch_util:get_value(<<"value">>, Doc),' +
-          '  (Value rem 2) =:= 0' +
+          '  case couch_util:get_value(<<"value">>, Doc) of' +
+          '  undefined -> false;' +
+          '  Value -> (Value rem 2) =:= 0;' +
+          '  _ -> false' +
+          '  end ' +
           'end.'
       }
     };
@@ -404,7 +423,7 @@ couchTests.changes = function(debug) {
 
     var options = {
         headers: {"Content-Type": "application/json"},
-        body: JSON.stringify({"doc_ids": ["something", "anotherthing", "andmore"]})         
+        body: JSON.stringify({"doc_ids": ["something", "anotherthing", "andmore"]})
     };
 
     var req = CouchDB.request("POST", "/test_suite_db/_changes?filter=_doc_ids", options);
@@ -447,7 +466,6 @@ couchTests.changes = function(debug) {
         
         T(db.save({"_id":"andmore", "bop" : "plankton"}).ok);
 
-        
         waitForSuccess(function() {
             if (xhr.readyState != 4) {
               throw("still waiting");
@@ -458,7 +476,6 @@ couchTests.changes = function(debug) {
         T(line.seq == 8);
         T(line.id == "andmore");
     }
-    
   });
 
   // COUCHDB-1037 - empty result for ?limit=1&filter=foo/bar in some cases
